#!/bin/bash

# Initialise variables

state=null
def_no=0

# Get the routers hostname
router_name=$(/bin/hostname)

echo
echo "Router $router_name dual link monitor"
echo '***************************'
echo "Diarmuid O'Briain"
echo

if [[ "$router_name" == 'n2' ]]; then
    MTN_IP='10.10.10.6'
    MTN_NextHop='10.10.10.1'
    MTN_NextNet='10.10.10.4/30'
    UTL_IP='10.10.10.14'
    UTL_NextHop='10.10.10.10'
    UTL_NextNet='10.10.10.12/30'
elif [[ "$router_name" == 'n3' ]]; then
    MTN_IP='10.10.10.2'
    MTN_NextHop='10.10.10.5'
    MTN_NextNet='10.10.10.0/30'
    UTL_IP='10.10.10.9'
    UTL_NextHop='10.10.10.13'
    UTL_NextNet='10.10.10.8/30'
else
    echo "This is $router_name and it is not one of the enterprise routers"
fi


while [ 1 ]; do

    sleep 5

    # Test links

    MTN_test=$(/usr/bin/fping -I  eth1 "$MTN_IP" | grep alive)
    UTL_test=$(/usr/bin/fping -I  eth2 "$UTL_IP" | grep alive)

    if [[ "$state" == 'null' ]]; then
        echo ">>> Attempting to determine link state"
    fi

    # // 4 possible states //
    # State 0 : Both links are up
    # State 1 : MTN up, UTL down
    # State 2 : UTL up, MTN down
    # State 3 : Both links are down

    # State 0 handler
    if [[ "$MTN_test" == *"alive"* && "$UTL_test" == *"alive"* ]]; then
        #echo "State 0 handler"
        if [[ "$state" == '0' ]]; then
            echo ">>> State 0 : Both links are up"
        else
            def_no=$(/sbin/ip route list | grep "default" | wc -l)
            until [[ "$def_no" == '0' ]]; do
                 /sbin/ip route del default
                 ((def_no--))
            done
            /sbin/ip route add default nexthop via "$MTN_NextHop" weight 1 nexthop via "$UTL_NextHop" weight 2
            echo ">>> Added multi-path route via both MTN and UTL"
            echo ">>> State 0 : Both links are up" 
            state=0     
        fi

    # State 1 handler
    elif [[ "$MTN_test" == *"alive"* && "$UTL_test" != *"alive"* ]]; then
        if [[ "$state" == '1' ]]; then
            echo ">>> State 1 : MTN up, UTL down, monitoring"
        else
            def_no=$(/sbin/ip route list | grep "default" | wc -l)
            until [[ "$def_no" == '0' ]]; do
                 /sbin/ip route del default
                 ((def_no--))
            done
            /sbin/ip route add default via "$MTN_NextHop" 
            /sbin/ip route add "$MTN_NextNet" via "$MTN_NextHop"
            /sbin/ip route add "$UTL_NextNet" via "$UTL_NextHop"
            echo ">>> Added bias route via MTN, diversity is lost"
            echo ">>> State 1 : MTN up, UTL down, monitoring"   
            state=1  
        fi
      
    # State 2 handler
    elif [[ "$MTN_test" != *"alive"* && "$UTL_test" == *"alive"* ]]; then
        if [[ "$state" == '2' ]]; then
            echo ">>> State 2 : UTL up, MTN down, monitoring"
        else
            def_no=$(/sbin/ip route list | grep "default" | wc -l)
            until [[ "$def_no" == '0' ]]; do
                 /sbin/ip route del default
                 ((def_no--))
            done
            /sbin/ip route add default via "$UTL_NextHop" 
            /sbin/ip route add "$MTN_NextNet" via "$MTN_NextHop"
            /sbin/ip route add "$UTL_NextNet" via "$UTL_NextHop"
            echo ">>> Added bias route via UTL, diversity is lost"
            echo ">>> State 2 : UTL up, MTN down, monitoring"   
            state=2
        fi

    # State 3 handler

    else
        if [[ "$state" == '3' ]]; then
            echo ">>> State 3 : UTL down, MTN down, panic, monitoring"
        else
            def_no=$(/sbin/ip route list | grep "default" | wc -l)
            until [[ "$def_no" == '0' ]]; do
                 /sbin/ip route del default
                 ((def_no--))
            done
            /sbin/ip route add "$MTN_NextNet" via "$MTN_NextHop"
            /sbin/ip route add "$UTL_NextNet" via "$UTL_NextHop"
            echo ">>> Trying to add routes to MTN and UTL, link is lost"
            echo ">>> State 3 : UTL down, MTN down, panic, monitoring"   
            state=3
        fi

    fi

done

# End

